/*
 * Copyright Contributors to the fusa-gcc-plugin project
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 */

#include "candidate.hpp"

#include <vector>

#include "gcc-plugin.h"
#include "input.h"

#include "cryptoutils.hpp"


using namespace std;

bool candidate::operator==(const candidate &other) const {
  if (api_type == API_TYPE_MACRO || other.api_type == API_TYPE_MACRO) {
    return this->name == other.name &&
           this->decl_source_file == other.decl_source_file;
  }

  return this->name == other.name &&
         this->decl_source_file == other.decl_source_file &&
         this->source_location == other.source_location;
}

bool candidate::fusa_check(vector<certified_symbol> &certified_symbols,
                           candidate::set &apis, struct plugin_args &args) const {
  if (args.profile == "qm") {
    if (args.dump_apis) {
      apis.insert(*this);
    }

    return true;
  }

  for (certified_symbol &symbol : certified_symbols) {
    if (this->name == symbol.name) {
      bool hash_equals =
          symbol.file_sha256 == crypto::sha256_file(this->decl_source_file);
      bool file_matches = symbol.file == this->decl_source_file;
      bool profile_matches = symbol.profiles.count(args.profile) == 1;

      bool verified = hash_equals && file_matches && profile_matches;

      if (verified)
        return true;
    }
  }

  return false;
}
