/*
 * Copyright Contributors to the fusa-gcc-plugin project
 *
 * SPDX-License-Identifier: LGPL-2.1-or-later
 */
#pragma once

#include <vector>

#include "gcc-plugin.h"
#include "tree.h"

using namespace std;

namespace declhelpers {
#define DECL_NAME_STRING(D) (IDENTIFIER_POINTER(DECL_NAME(D)))
#define DECL_SET_IF(T, V)                                                      \
  ({                                                                           \
    if (TREE_CODE(decl) == (T))                                                \
      decl = V(decl);                                                          \
  })
#define DECL_SET_IF_ELSE(T, V, N)                                              \
  ({                                                                           \
    if (TREE_CODE(decl) == (T))                                                \
      decl = V(decl);                                                          \
    else                                                                       \
      decl = N(decl);                                                          \
  })

// Get namespace hierarchy for the given decl
//
//   std::algorithm::find_if => ["std", "algorithm"]
//
vector<string> get_namespace_hierarchy(tree decl);

// Get class hierarchy for the given decl
//
//   std::basic_string::allocator => ["basic_string", "allocator"]
//
vector<string> get_class_hierarchy(tree decl);

// Recursively iterate the tree `t` and return all nodes
// of the FUNCTION_DECL type
vector<tree> find_function_decls(tree decl);
} // namespace declhelpers
